@@ -26,6 +26,10 @@
 #if defined(_MSC_VER)
 #  pragma warning(push)
 #  pragma warning(disable: 4127) // warning C4127: Conditional expression is constant
+#ifdef _WIN64
+#include <BaseTsd.h>
+using ssize_t = SSIZE_T;
+#endif
 #endif
 
 /* This will be true on all flat address space platforms and allows us to reduce the
