@@ -11,10 +11,12 @@
 #include "image_loader/image_loader.h"
 #include "image_loader/png_loader.h"
 #include "image_loader/jpeg_loader.h"
+#include "image_loader/webp_loader.h"
 #include "image_loader/load_image.h"
 #include "image_saver/image_saver.h"
 #include "image_saver/save_png.h"
 #include "image_saver/save_jpeg.h"
+#include "image_saver/save_webp.h"
 
 #endif // DLIB_IMAGe_IO_ 
 
