@@ -39,6 +39,7 @@ namespace dlib
             - bool lab
 
             - bool has_alpha
+            - bool bgr_layout
 
             - long num 
 
@@ -55,6 +56,7 @@ namespace dlib
                 - This type of pixel represents the RGB color space.
                 - num == 3
                 - has_alpha == false
+                - bgr_layout == true if the channel order is BGR, and false if it's RGB
                 - basic_pixel_type == unsigned char
                 - min() == 0 
                 - max() == 255
@@ -68,6 +70,7 @@ namespace dlib
                   with maximum opacity.
                 - num == 4
                 - has_alpha == true 
+                - bgr_layout == true if the channel order is BGR, and false if it's RGB
                 - basic_pixel_type == unsigned char
                 - min() == 0 
                 - max() == 255
@@ -132,6 +135,12 @@ namespace dlib
                 && this->green == that.green
                 && this->blue  == that.blue;
         }
+
+        bool operator != (const rgb_pixel& that) const
+        {
+            return !(*this == that);
+        }
+
     };
 
 // ----------------------------------------------------------------------------------------
@@ -165,6 +174,12 @@ namespace dlib
                 && this->green == that.green
                 && this->red   == that.red;
         }
+
+        bool operator != (const bgr_pixel& that) const
+        {
+            return !(*this == that);
+        }
+
     };
 
 // ----------------------------------------------------------------------------------------
@@ -199,6 +214,52 @@ namespace dlib
                 && this->blue  == that.blue
                 && this->alpha == that.alpha;
         }
+
+        bool operator != (const rgb_alpha_pixel& that) const
+        {
+            return !(*this == that);
+        }
+
+    };
+
+// ----------------------------------------------------------------------------------------
+
+    struct bgr_alpha_pixel
+    {
+        /*!
+            WHAT THIS OBJECT REPRESENTS
+                This is a simple struct that represents an BGR colored graphical pixel
+                with an alpha channel.
+        !*/
+
+        bgr_alpha_pixel (
+        ) {}
+
+        bgr_alpha_pixel (
+            unsigned char blue_,
+            unsigned char green_,
+            unsigned char red_,
+            unsigned char alpha_
+        ) : blue(blue_), green(green_), red(red_), alpha(alpha_) {}
+
+        unsigned char blue;
+        unsigned char green;
+        unsigned char red;
+        unsigned char alpha;
+
+        bool operator == (const bgr_alpha_pixel& that) const
+        {
+            return this->blue == that.blue
+                && this->green == that.green
+                && this->red == that.red
+                && this->alpha == that.alpha;
+        }
+
+        bool operator != (const bgr_alpha_pixel& that) const
+        {
+            return !(*this == that);
+        }
+
     };
 
 // ----------------------------------------------------------------------------------------
@@ -229,7 +290,14 @@ namespace dlib
                 && this->s == that.s
                 && this->i == that.i;
         }
+
+        bool operator != (const hsi_pixel& that) const
+        {
+            return !(*this == that);
+        }
+
     };
+
     // ----------------------------------------------------------------------------------------
 
     struct lab_pixel
@@ -258,7 +326,13 @@ namespace dlib
                 && this->a == that.a
                 && this->b == that.b;
         }
-    };
+
+        bool operator != (const lab_pixel& that) const
+        {
+            return !(*this == that);
+        }
+
+    };    
 
 // ----------------------------------------------------------------------------------------
 
@@ -443,6 +517,7 @@ namespace dlib
     struct pixel_traits<rgb_pixel>
     {
         constexpr static bool rgb  = true;
+        constexpr static bool bgr_layout  = false;
         constexpr static bool rgb_alpha  = false;
         constexpr static bool grayscale = false;
         constexpr static bool hsi = false;
@@ -461,6 +536,7 @@ namespace dlib
     struct pixel_traits<bgr_pixel>
     {
         constexpr static bool rgb  = true;
+        constexpr static bool bgr_layout  = true;
         constexpr static bool rgb_alpha  = false;
         constexpr static bool grayscale = false;
         constexpr static bool hsi = false;
@@ -479,6 +555,26 @@ namespace dlib
     struct pixel_traits<rgb_alpha_pixel>
     {
         constexpr static bool rgb  = false;
+        constexpr static bool bgr_layout  = false;
+        constexpr static bool rgb_alpha  = true;
+        constexpr static bool grayscale = false;
+        constexpr static bool hsi = false;
+        constexpr static bool lab = false;
+        constexpr static long num = 4;
+        typedef unsigned char basic_pixel_type;
+        static basic_pixel_type min() { return 0;}
+        static basic_pixel_type max() { return 255;}
+        constexpr static bool is_unsigned = true;
+        constexpr static bool has_alpha = true;
+    };
+
+// ----------------------------------------------------------------------------------------
+
+    template <>
+    struct pixel_traits<bgr_alpha_pixel>
+    {
+        constexpr static bool rgb  = false;
+        constexpr static bool bgr_layout = true;
         constexpr static bool rgb_alpha  = true;
         constexpr static bool grayscale = false;
         constexpr static bool hsi = false;
@@ -498,6 +594,7 @@ namespace dlib
     struct pixel_traits<hsi_pixel>
     {
         constexpr static bool rgb  = false;
+        constexpr static bool bgr_layout  = false;
         constexpr static bool rgb_alpha  = false;
         constexpr static bool grayscale = false;
         constexpr static bool hsi = true;
@@ -517,6 +614,7 @@ namespace dlib
     struct pixel_traits<lab_pixel>
     {
         constexpr static bool rgb  = false;
+        constexpr static bool bgr_layout  = false;
         constexpr static bool rgb_alpha  = false;
         constexpr static bool grayscale = false;
         constexpr static bool hsi = false;
@@ -535,6 +633,7 @@ namespace dlib
     struct grayscale_pixel_traits
     {
         constexpr static bool rgb  = false;
+        constexpr static bool bgr_layout  = false;
         constexpr static bool rgb_alpha  = false;
         constexpr static bool grayscale = true;
         constexpr static bool hsi = false;
@@ -679,6 +778,12 @@ namespace dlib
         }
     // -----------------------------
 
+        // Apparently Visual Studio's optimizer complains about branches that would not be taken anyway.
+#if defined (_MSC_VER)
+#pragma warning ( push )
+#pragma warning ( disable: 4756 )
+#endif
+
         template < typename P1, typename P2 >
         typename enable_if_c<pixel_traits<P1>::grayscale && pixel_traits<P2>::grayscale>::type
         assign(P1& dest, const P2& src) 
@@ -697,6 +802,10 @@ namespace dlib
                 dest = pixel_traits<P1>::max();
         }
 
+#if defined (_MSC_VER)
+#pragma warning ( pop )
+#endif
+
     // -----------------------------
     // -----------------------------
     // -----------------------------
@@ -978,9 +1087,9 @@ namespace dlib
             }
 
             //clamping
-            c2.l = max(0.0, (116.0 * var_Y) - 16);
-            c2.a = max(-128.0, min(127.0, 500.0 * (var_X - var_Y)));
-            c2.b = max(-128.0, min(127.0, 200.0 * (var_Y - var_Z)));
+            c2.l = std::max(0.0, (116.0 * var_Y) - 16);
+            c2.a = std::max(-128.0, std::min(127.0, 500.0 * (var_X - var_Y)));
+            c2.b = std::max(-128.0, std::min(127.0, 200.0 * (var_Y - var_Z)));
 
             return c2;
         }
@@ -1049,9 +1158,9 @@ namespace dlib
             }
 
             // clamping
-            c2.r = max(0.0, min(1.0, var_R));
-            c2.g = max(0.0, min(1.0, var_G));
-            c2.b = max(0.0, min(1.0, var_B));
+            c2.r = std::max(0.0, std::min(1.0, var_R));
+            c2.g = std::max(0.0, std::min(1.0, var_G));
+            c2.b = std::max(0.0, std::min(1.0, var_B));
 
             return (c2);
         }
